import { message } from 'antd';
import { query, create, update, remove } from '../../services/product';
import modelExtend from 'dva-model-extend';
import queryString from 'query-string';
import { pageModel } from '../common';
import { pageSize, Codes } from '../../utils/config';
import { checkSearchParams } from '../../utils/utils';

export default modelExtend(pageModel, {
  namespace: 'combo',

  state: {
    currentItem: {},
    itemLoading: false,
    listLoading: false,
    modalVisible: false,
    modalType: 'create',
  },

  subscriptions: {
    setup({ dispatch, history }) {
      history.listen((location) => {
        if (location.pathname === '/product/package') {
          const payload = checkSearchParams(queryString.parse(location.search));
          dispatch({
            type: 'query',
            payload,
          });
        }
      });
    }
  },

  effects: {
    * query ({ payload = {} }, { call, put }) {
      yield put({ type: 'changeLoading', payload: { listLoading: true }});
      const { data, success } = yield call(query, { ...payload, type: Codes.CODE_PACKAGE });
      if (success) {
        yield put({
          type: 'querySuccess',
          payload: {
            list: data.list,
            pagination: {
              current: Number(payload.pageNo) || 1,
              pageSize: Number(payload.pageSize) || pageSize,
              total: data.totalSize,
            }
          }
        });
      }
      yield put({ type: 'changeLoading', payload: { listLoading: false }});
    },
    * delete ({ payload, callback }, { call, put }) {
      const { data, success } = yield call(remove, payload);
      if (success) {
        message.success('删除成功!');
        if (callback) callback();
      }
    },
  },

  reducers: {
    changeLoading(state, action) {
      return { ...state, ...action.payload };
    },
    showModal(state, action) {
      return { ...state, ...action.payload, modalVisible: true };
    },
    hideModal(state, action) {
      return { ...state, ...action.payload, modalVisible: false };
    },
  }
})
