import { queryOne, create, update } from '../../services/group';
import pathToRegexp from 'path-to-regexp';
import { message } from 'antd';
import { Codes } from '../../utils/config';

export default {
  namespace: 'groupDetail',

  state: {
    filters: {},
    operType: 'create',
    currentItem: {},
    modalVisible: false,
    itemLoading: false,
  },

  subscriptions: {
    setup({ dispatch, history }) {
      history.listen(({ pathname, state }) => {
        const match = pathToRegexp('/tag/tagGroup/edit/:id').exec(pathname);
        if (match) {
          dispatch({ type: 'query', payload: { id: match[1] } });
          dispatch({ type: 'saveFilters', payload: state });
          dispatch({ type: 'saveOperType', payload: { operType: 'update' } });
        }
        if (pathname == '/tag/tagGroup/add') {
          dispatch({ type: 'saveFilters', payload: state });
          dispatch({ type: 'saveOperType', payload: { operType: 'create' } });
        }
      });
    }
  },

  effects: {
    * query ({ payload }, { call, put }) {
      yield put({ type: 'changeLoading', payload: { itemLoading: true } });
      const { data, success } = yield call(queryOne, payload);
      if (success) {
        yield put({ type: 'querySuccess', payload: { ...data } });
      }
      yield put({ type: 'changeLoading', payload: { itemLoading: false } });
    },
    * create ({ payload, callback }, { call, put }) {
      const { data, success } = yield call(create, { ...payload, status: Codes.CODE_NORMAL });
      if (success) {
        message.success('创建成功!');
        if (callback) callback();
      }
    },
    * update ({ payload, callback }, { call, put }) {
      const { data, success } = yield call(update, payload);
      if (success) {
        message.success('修改成功!');
        if (callback) callback();
      }
    }
  },

  reducers: {
    changeLoading(state, action) {
      return { ...state, ...action.payload };
    },
    querySuccess(state, action) {
      const currentItem = action.payload;
      return { ...state, currentItem };
    },
    saveFilters(state, action) {
      const filters = action.payload;
      return { ...state, filters };
    },
    showModal(state, action) {
      return { ...state, ...action.payload, modalVisible: true };
    },
    hideModal(state, action) {
      return { ...state, ...action.payload, modalVisible: false };
    },
    saveOperType(state, action) {
      return { ...state, ...action.payload };
    },
    saveSortResult(state, action) {
      const tagList = action.payload.tagList;
      const currentItem = { ...state.currentItem, tagList };
      return { ...state, modalVisible: false, currentItem };
    },
    initState(state) {
      return { ...state, currentItem: {}, itemLoading: false };
    }
  }
}
