import modelExtend from 'dva-model-extend';
import queryString from 'query-string';
import { message } from 'antd';
import { query, remove, moneyCharge } from '../../services/merchant';
import { pageModel } from '../common';
import config from '../../utils/config';
import { checkSearchParams } from '../../utils/utils';

export default modelExtend(pageModel, {
  namespace: 'merchant',

  state: {
    listLoading: false,
    modalVisible: false,
    currentItem: {},
  },

  subscriptions: {
    setup({ dispatch, history }) {
      history.listen((location) => {
        if (location.pathname === '/merchant') {
          const payload = checkSearchParams(queryString.parse(location.search));
          dispatch({
            type: 'query',
            payload,
          });
        }
      });
    },
  },

  effects: {
    * query({ payload = {} }, { call, put }) {
      yield put({ type: 'changeLoading', payload: { listLoading: true } });
      const { data, success } = yield call(query, payload);
      if (success) {
        yield put({
          type: 'querySuccess',
          payload: {
            list: data.list,
            pagination: {
              current: Number(payload.pageNo) || 1,
              pageSize: Number(payload.pageSize) || config.pageSize,
              total: data.totalSize,
            },
          },
        });
      }
      yield put({ type: 'changeLoading', payload: { listLoading: false } });
    },
    * delete({ payload, callback }, { call }) {
      const { success } = yield call(remove, payload);
      if (success) {
        message.success('删除成功!');
        if (callback) callback();
      }
    },
    * recharge({ payload, callback }, { call, put }) {
      const { success } = yield call(moneyCharge, payload);
      if (success) {
        message.success('充值成功!');
        yield put({ type: 'hideModal', payload: {} });
        if (callback) callback();
      }
    },
  },

  reducers: {
    changeLoading(state, action) {
      return { ...state, ...action.payload };
    },
    showModal(state, action) {
      return {
        ...state,
        modalVisible: true,
        currentItem: action.payload,
      };
    },
    hideModal(state, action) {
      return { ...state, ...action.payload, modalVisible: false };
    },
  },
});
