import { query, update, createMerchantProduct, queryFromShopCart } from '../../services/mproduct';
import modelExtend from 'dva-model-extend';
import queryString from 'query-string';
import { message } from 'antd';
import { pageModel } from '../common';
import { pageSize, Codes } from '../../utils/config';
import { checkSearchParams } from '../../utils/utils';

export default modelExtend(pageModel, {
  namespace: 'mproduct',

  state: { listLoading: false },

  subscriptions: {
    setup({ dispatch, history }) {
      history.listen((location) => {
        if (location.pathname == '/goods') {
          const payload = checkSearchParams(queryString.parse(location.search));
          dispatch({
            type: 'query',
            payload,
          });
        }
      });
    }
  },

  effects: {
    * query ({ payload = {} }, { call, put }) {
      yield put({ type: 'changeLoading', payload: { listLoading: true }});
      const { data, success } = yield call(query, payload);
      if (success) {
        yield put({
          type: 'querySuccess',
          payload: {
            list: data.list,
            pagination: {
              current: Number(payload.pageNo) || 1,
              pageSize: Number(payload.pageSize) || pageSize,
              total: data.totalSize,
            }
          }
        });
      }
      yield put({ type: 'changeLoading', payload: { listLoading: false }});
    },
    * queryBuyMsg({payload}, {call, put}) {
      yield put({ type: 'changeLoading', payload: { listLoading: true }});
      const { data, success } = yield call(queryFromShopCart, payload);
      if (success) {
        yield put({
          type: 'querySuccess',
          payload: {
            list: data || [],
            pagination: {
              current: Number(payload.pageNo) || 1,
              pageSize: Number(payload.pageSize) || pageSize,
              total: data.totalSize,
            }
          }
        });
      }
      yield put({ type: 'changeLoading', payload: { listLoading: false }});
    },
    // 商品上架 - {status: NORMAL, merchantId: 'xxx', pid: 'xxx'}
    * putOnSale ({ payload, callback }, { call, put }) {
      const { data, success } = yield call(createMerchantProduct, { ...payload, status: Codes.CODE_NORMAL });
      if (success) {
        message.success('上架成功!');
        if (callback) callback();
      }
    },
    // 商品下架 - {status: 'DEL', merchantId: 'xxx', pid: 'xxx'}
    * putOffSale ({ payload, callback }, { call, put }) {
      const { data, success } = yield call(createMerchantProduct, { ...payload, status: Codes.CODE_DELETE });
      if (success) {
        message.success('下架成功!');
        if (callback) callback();
      }
    },
  },

  reducers: {
    changeLoading(state, { payload }) {
      return { ...state, ...payload };
    },
  }
})
