import { message } from 'antd';
import { queryOne, create, update } from '../../services/product';
import pathToRegexp from 'path-to-regexp';
import { Codes } from '../../utils/config';

export default {
  namespace: 'trainingDetail',

  state: {
    filters: {},
    operType: 'create',
    currentItem: {},
    resourceModalVisible: false,
    itemLoading: false,
  },

  subscriptions: {
    setup({ dispatch, history }) {
      history.listen(({ pathname, state, ...rest }) => {
        const match = pathToRegexp('/product/training/edit/:pid').exec(pathname);
        if (match) {
          dispatch({ type: 'query', payload: { pid: match[1] } });
          dispatch({ type: 'saveFilters', payload: state });
          dispatch({ type: 'saveOperType', payload: { operType: 'update' } });
        }
        if (pathname == '/product/training/add') {
          dispatch({ type: 'saveFilters', payload: state });
          dispatch({ type: 'saveFilters', payload: state });
          dispatch({ type: 'saveOperType', payload: { operType: 'create' } });
        }
      });
    }
  },

  effects: {
    * query ({ payload }, { call, put }) {
      yield put({ type: 'changeLoading', payload: { itemLoading: true } });
      const { data, success } = yield call(queryOne, payload);
      if (success) {
        yield put({ type: 'querySuccess', payload: data });
      }
      yield put({ type: 'changeLoading', payload: { itemLoading: false } });
    },
    * create ({ payload, callback }, { call, put }) {
      const { data, success } = yield call(create, payload);
      if (success) {
        message.success('创建成功!');
        if (callback) callback();
        yield put({ type: 'initState' });
      }
    },
    * update ({ payload, callback }, { call, put }) {
      const { data, success } = yield call(update, payload);
      if (success) {
        message.success('修改成功!');
        if (callback) callback();
        yield put({ type: 'initState' });
      }
    }
  },

  reducers: {
    changeLoading(state, action) {
      return { ...state, ...action.payload };
    },
    querySuccess(state, action) {
      return {
        ...state,
        currentItem: action.payload,
      };
    },
    saveFilters(state, action) {
      return {
        ...state,
        filters: action.payload.filters,
      };
    },
    showResourceModal(state, action) {
      return {
        ...state,
        ...action.payload,
        resourceModalVisible: true,
      };
    },
    hideResourceModal(state, action) {
      return {
        ...state,
        ...action.payload,
        resourceModalVisible: false,
      };
    },
    saveCoverImg(state, action) {
      return {
        ...state,
        resourceModalVisible: false,
        currentItem: {
          ...state.currentItem,
          coverUrl: action.payload.coverUrl,
        },
      };
    },
    saveImgList(state, action) {
      return {
        ...state,
        resourceModalVisible: false,
        currentItem: {
          ...state.currentItem,
          imgList: action.payload.imgList,
        }
      };
    },
    saveOperType(state, action) {
      return { ...state, ...action.payload };
    },
    initState(state) {
      return { ...state, currentItem: {}, itemLoading: false };
    }
  }
}
