import { create, update, queryOne } from '../../services/whitelist';
import pathToRegexp from 'path-to-regexp';
import { message } from 'antd';

export default {
  namespace: 'whitelistDetail',

  state: {
    filters: {},
    operType: 'create',
    currentItem: {},
    modalShow: false,
  },

  subscriptions: {
    setup({ dispatch, history }) {
      history.listen(({ pathname, state, record }) => {
        const match = pathToRegexp('/terminal/whitelist/edit/:id').exec(pathname);
        if (match) {
          dispatch({
            type: 'saveOuterData',
            payload: { filters: state, operType: 'update'},
          });
          dispatch({
            type: 'queryOne',
            payload: {userId: match[1]},
          });
        } else if (pathname === '/terminal/whitelist/add') {
          dispatch({
            type: 'saveOuterData',
            payload: { filters: state, operType: 'create', currentItem: {} },
          });
        }
      });
    }
  },

  effects: {
    * queryOne({ payload, callback }, { call, put }) {
      const { data, success } = yield call(queryOne, payload);
      if (success) {
        yield put({
          type: 'saveCurrentItem',
          payload: {...data},
        });
      }
    },
    * create ({ payload, callback }, { call, put }) {
      const { data, success } = yield call(create, payload);
      if (success) {
        message.success('创建成功!');
        yield put({ type: 'initState' });
        if (callback) callback();
      }
    },
    * update ({ payload, callback }, { call, put }) {
      const { data, success } = yield call(update, payload);
      if (success) {
        message.success('修改成功!');
        yield put({ type: 'initState' });
        if (callback) callback();
      }
    },
  },

  reducers: {
    saveOuterData(state, action) {
      return { ...state, ...action.payload };
    },
    saveCurrentItem(state, action) {
      return {
        ...state,
        currentItem: action.payload,
      };
    },
    showModal(state) {
      return { ...state, modalShow: true };
    },
    hideModal(state, action) {
      const { currentItem } = state;
      return {
        ...state,
        modalShow: false,
        currentItem: {
          ...currentItem,
          ...action.payload,
        }
      };
    },
    initState(state) {
      return { ...state, currentItem: {} };
    }
  }
}
