import React, { Component } from 'react';
import queryString from 'query-string';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Card } from 'antd';
import CMSUserTableList from './table';
import CMSUserModalForm from './modal';
import CMSUserSearch from './search';
import PageHeaderLayout from '../../layouts/PageHeaderLayout';

@connect(state => ({
  cmsUser: state.cmsUser,
  merchant: state.merchant,
}))
export default class CMSUser extends Component {
  componentDidMount() {
    this.props.dispatch({
      type: 'merchant/query',
      payload: {
        pageNo: 1,
        pageSize: 1000,
      },
    });
  }

  render() {
    const { location, dispatch, cmsUser, merchant } = this.props;

    location.query = queryString.parse(location.search);
    const { query, pathname } = location;
    const { field, keyword, ...filters } = query;
    const { list, listLoading, pagination, currentItem, modalVisible, modalType } = cmsUser;

    Object.keys(filters).map(key => { filters[key] ? null : delete filters[key] });
    if (field && keyword) {
      filters.field = field;
      filters.keyword = keyword;
    }

    const modalProps = {
      item: modalType === 'create' ? {} : currentItem,
      visible: modalVisible,
      merchantList: merchant.list,
      maskClosable: false,
      modalType,
      title: `${modalType === 'create' ? '添加CMS用户' : '编辑CMS用户'}`,
      wrapClassName: 'vertical-center-modal',
      onOk(data) {
        dispatch({
          type: `cmsUser/${modalType}`,
          payload: data,
          callback: () => {
            dispatch(routerRedux.push({
              pathname: '/cms/user',
              search: queryString.stringify(filters),
            }));
          },
        });
      },
      onCancel() {
        dispatch({
          type: 'cmsUser/hideModal',
        });
      },
    };

    const searchProps = {
      field,
      keyword,
      onSearch: (payload) => {
        if (!payload.keyword.length) {
          delete payload.field;
          delete payload.keyword;
        }
        dispatch(routerRedux.push({
          pathname,
          search: queryString.stringify({
            ...payload,
          }),
        }));
      },
      onAdd: () => {
        dispatch({
          type: 'cmsUser/showModal',
          payload: {
            modalType: 'create',
          },
        });
      },
    };

    const listProps = {
      location,
      pagination,
      dataSource: list,
      loading: listLoading,
      curStatus: filters.status,
      onChange: (pagination, filterArgs) => {
        const getValue = obj => Object.keys(obj).map(key => obj[key]).join(',');
        const tableFilters = Object.keys(filterArgs).reduce((obj, key) => {
          const newObj = { ...obj };
          newObj[key] = getValue(filterArgs[key]);
          return newObj;
        }, {});

        const data = { ...filters, ...tableFilters };
        Object.keys(data).map(key => data[key] ? null : delete data[key]);
        dispatch(routerRedux.push({
          pathname,
          search: queryString.stringify({
            ...data,
            pageNo: pagination.current,
            pageSize: pagination.pageSize,
          }),
        }));
      },
      // 编辑
      onEditItem: (item) => {
        dispatch({
          type: 'cmsUser/showModal',
          payload: {
            modalType: 'update',
            currentItem: item,
          },
        })
      },
      // 封禁
      onDeleteItem: (id) => {
        dispatch({
          type: 'cmsUser/delete',
          payload: id,
          callback: () => {
            dispatch(
              routerRedux.push({
                pathname,
                search: queryString.stringify(filters),
              })
            );
          },
        });
      },
      // 解禁
      onRecoverItem: (payload) => {
        dispatch({
          type: 'cmsUser/recover',
          payload,
          callback: () => {
            dispatch(
              routerRedux.push({
                pathname,
                search: queryString.stringify(filters),
              })
            );
          },
        });
      },
    };

    return (
      <PageHeaderLayout>
        <Card>
          <CMSUserSearch {...searchProps} />
          <CMSUserTableList {...listProps} />
          {modalVisible && <CMSUserModalForm {...modalProps} />}
        </Card>
      </PageHeaderLayout>
    );
  }
}
