import React, { PureComponent } from 'react';
import { routerRedux } from 'dva/router';
import queryString from 'query-string';
import { connect } from 'dva';
import { Spin, Table, Card, Form, Input, Icon, Button, Select } from 'antd';
import PageHeaderLayout from '../../../layouts/PageHeaderLayout';
import LessonSelectSortModal from './lesson';
import ResourceSelectModal from './resource';
import SupportSelectSortModal from './support';
import { Codes, ossHost } from '../../../utils/config';

@Form.create()
@connect(state => ({
  courseDetail: state.courseDetail,
  resource: state.resource,
  lesson: state.lesson,
  support: state.support,
  merchant: state.merchant,
}))
export default class CourseDetail extends PureComponent {
  state = {
    curClickedBtn: null,       //记录点击的按钮
  };

  componentDidMount() {
    const { dispatch } = this.props;
    dispatch({
      type: 'merchant/query',
      payload: {
        pageSize: 1000,
        pageNo: 1,
      }
    });
  }

  // 展示选择模态框 - 加载第一页数据
  handleModalShow = (btnName) => {
    this.setState({
      curClickedBtn: btnName,
    }, () => {
      const { dispatch } = this.props;
      if (btnName == 'lessonBtn') {
        dispatch({ type: 'courseDetail/showLessonModal' });
        dispatch({
          type: 'lesson/query',
          payload: {
            pageNo: 1,
            pageSize: 10,
            status: Codes.CODE_NORMAL,
          }
        });
      } else if (btnName == 'cvImgBtn' || btnName == 'bgImgBtn') {
        dispatch({ type: 'courseDetail/showResourceModal' });
        dispatch({
          type: 'resource/query',
          payload: {
            pageNo: 1,
            pageSize: 10,
            status: Codes.CODE_NORMAL,
            type: Codes.CODE_IMAGE,
          }
        });
      } else if (btnName == 'supportBtn') {
        dispatch({ type: 'courseDetail/showSupportModal' });
        dispatch({
          type: 'support/query',
          payload: {
            pageNo: 1,
            pageSize: 10,
            status: Codes.CODE_NORMAL,
          }
        });
      }
    })
  }

  // 取消/关闭 - 隐藏选择模态框
  handleModalCancel = () => {
    const { curClickedBtn } = this.state;
    const { dispatch } = this.props;
    if (curClickedBtn == 'lessonBtn') {
      dispatch({ type: 'courseDetail/hideLessonModal' });
    } else if (curClickedBtn == 'cvImgBtn' || curClickedBtn == 'bgImgBtn') {
      dispatch({ type: 'courseDetail/hideResourceModal' });
    } else if (curClickedBtn == 'supportBtn') {
      dispatch({ type: 'courseDetail/hideSupportModal' });
    }
  }

  // 提交 - 保存选择和排序完的数据到model中
  handleModalOk = (data) => {
    const { curClickedBtn } = this.state;
    const { dispatch } = this.props;
    if (curClickedBtn == 'lessonBtn') {
      dispatch({
        type: 'courseDetail/saveLessonList',
        payload: { subItemList: data }
      });
    } else if (curClickedBtn == 'cvImgBtn') {
      dispatch({
        type: 'courseDetail/saveCoverImg',
        payload: { coverUrl: data.path },
      });
    } else if (curClickedBtn == 'bgImgBtn') {
      dispatch({
        type: 'courseDetail/saveBackgroundImg',
        payload: { bgUrl: data.path },
      });
    } else if (curClickedBtn == 'supportBtn') {
      dispatch({
        type: 'courseDetail/saveSupportList',
        payload: { supportList: data }
      });
    }
  }

  // 搜索
  handleModalSearch = (data) => {
    const { curClickedBtn } = this.state;
    const { dispatch } = this.props;
    const newData = { ...data };
    if (newData.keyword) {
      newData[newData.field] = newData.keyword;
    }
    delete newData.field;
    delete newData.keyword;

    if (curClickedBtn == 'lessonBtn') {
      dispatch({
        type: `lesson/query`,
        payload: { ...newData, pageNo: 1, pageSize: 10, status: Codes.CODE_NORMAL },
      });
    } else if (curClickedBtn == 'cvImgBtn' || curClickedBtn == 'bgImgBtn') {
      dispatch({
        type: `resource/query`,
        payload: { ...newData, pageNo: 1, pageSize: 10, status: Codes.CODE_NORMAL, type: Codes.CODE_IMAGE },
      });
    }
  }

  // 翻页 - 资源列表
  handleModalTableChange = (pagination, filterArgs, filters) => {
    const { curClickedBtn } = this.state;
    const { dispatch } = this.props;
    const newFilters = { ...filters };
    if (newFilters.keyword) {
      newFilters[newFilters.field] = newFilters.keyword;
    }
    delete newFilters.field;
    delete newFilters.keyword;

    const getValue = obj => Object.keys(obj).map(key => obj[key]).join(',');
    const tableFilters = Object.keys(filterArgs).reduce((obj, key) => {
      const newObj = { ...obj };
      newObj[key] = getValue(filterArgs[key]);
      return newObj;
    }, {});

    const data = { ...newFilters, ...tableFilters, pageNo: pagination.current, pageSize: pagination.pageSize };
    Object.keys(data).map(key => data[key] ? null : delete data[key]);

    if (curClickedBtn == 'lessonBtn') {
      dispatch({ type: `lesson/query`, payload: data });
    } else if (curClickedBtn == 'cvImgBtn' || curClickedBtn == 'bgImgBtn') {
      dispatch({ type: `resource/query`, payload: { ...data, type: Codes.CODE_IMAGE } });
    } else if (curClickedBtn == 'supportBtn') {
      dispatch({ type: `support/query`, payload: data });
    }
  }

  handlePageSubmit = (e) => {
    e.preventDefault()
    const {
      dispatch,
      form: {
        validateFields,
        getFieldsValue,
        resetFields
      },
      courseDetail: {
        operType,
        currentItem,
        filters,
      }
    } = this.props;
    validateFields((errors) => {
      if (errors) { return; }

      // 过滤掉gmtCreated, gmtModified
      const { subItemList, supportList, gmtCreated, gmtModified, ...rest } = currentItem;
      let newSubItemList;
      let newSupportList;
      // 如果subItemList是个list且内容不为空
      if (Array.isArray(subItemList) && subItemList.length) {
        newSubItemList = subItemList.map(item => ({ id: item.id, type: Codes.CODE_LESSON }));
      } else {
        newSubItemList = [];
      }
      // 如果supportList是个list且内容不为空
      if (Array.isArray(supportList) && supportList.length) {
        newSupportList = supportList.map(item => item.id);
      } else {
        newSupportList = [];
      }

      // 取出名称name值作为标题title字段值
      const { title, subTitle } = getFieldsValue();

      // 最终要提交的数据
      const data = {
        ...rest,
        ...getFieldsValue(),
        name: `${title}${subTitle}`,
        subItemList: newSubItemList,
        supportList: newSupportList,
      };

      // 加入状态/类型
      if (operType == 'create') {
        data.type = Codes.CODE_COURSE;
        data.status = Codes.CODE_NORMAL;
      }

      // 操作成功要返回列表页，同时要保持列表页的过滤数据
      dispatch({
        type: `courseDetail/${operType}`,
        payload: data,
        callback: () => {
          dispatch(
            routerRedux.push({
              pathname: '/product/course',
              search: queryString.stringify(filters),
            })
          );
          resetFields();
        }
      })
    });
  }

  handlePageCancel = () => {
    const { dispatch, courseDetail: { filters } } = this.props;
    dispatch(
      routerRedux.push({
        pathname: '/product/course',
        search: queryString.stringify(filters),
      })
    );
    dispatch({ type: 'courseDetail/initState' });
  }

  render() {
    const {
      dispatch,
      courseDetail,
      lesson,
      resource,
      support,
      merchant,
      form: {
        getFieldDecorator
      },
    } = this.props;
    const {
      itemLoading,
      currentItem,
      filters,
      lessonModalVisible,
      supportModalVisible,
      resourceModalVisible
    } = courseDetail;
    const {
      name,
      title,
      subTitle,
      breadCrumb,
      code,
      digest,
      detail,
      cpId,
      coverUrl,
      bgUrl,
      subItemList = [],
      supportList = [],
    } = currentItem;

    // 待选表格去掉分页的跳转及变换页码
    if (resource && resource.pagination) {
      delete resource.pagination.showQuickJumper;
      delete resource.pagination.showSizeChanger;
    }

    const lessonTableColumns = [{
      title: '课编号',
      dataIndex: 'code',
      key: 'code',
    },{
      title: '课名称',
      dataIndex: 'title',
      key: 'title',
    }];

    const supportTableColumns = [{
      title: '配套编号',
      dataIndex: 'code',
      key: 'code',
    },{
      title: '配套名称',
      dataIndex: 'name',
      key: 'name',
    }]

    const formItemLayout = {
      labelCol: {
        span: 7,
      },
      wrapperCol: {
        span: 12,
      },
    };
    const submitFormLayout = {
      wrapperCol: {
        xs: { span: 24, offset: 0 },
        sm: { span: 10, offset: 7 },
      },
    };

    return (
      <PageHeaderLayout>
        <Spin spinning={itemLoading}>
          <Card title="课程信息">
            <Form layout="horizontal" onSubmit={this.handlePageSubmit}>
              <Form.Item label="课程编号：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('code', {
                  rules: [{ required: true, type: 'string', message: "编号为必填项!" }],
                  initialValue: code,
                })(<Input />)}
              </Form.Item>
              <Form.Item label="课程标题：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('title', {
                  rules: [{ required: true, type: 'string', message: "标题为必填项!" }],
                  initialValue: title,
                })(<Input />)}
              </Form.Item>
              <Form.Item label="课程副标题：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('subTitle', {
                  initialValue: subTitle,
                })(<Input />)}
              </Form.Item>
              <Form.Item label="课程名称：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('name', {
                  initialValue: name,
              })(<Input disabled={true} placeholder="根据标题及副标题自动生成"/>)}
              </Form.Item>
              <Form.Item label="面包屑导航：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('breadCrumb', {
                  initialValue: breadCrumb,
                })(<Input />)}
              </Form.Item>
              <Form.Item label="课程概要：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('digest', {
                  initialValue: digest,
                })(<Input.TextArea />)}
              </Form.Item>
              <Form.Item label="课程详情：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('detail', {
                  initialValue: detail,
                })(<Input.TextArea />)}
              </Form.Item>
              <Form.Item label="所属供应商：" {...formItemLayout}>
                {getFieldDecorator('cpId', {
                  rules: [{ required: true, type: 'string', message: '供应商为必选项!' }],
                  initialValue: cpId,
                })(
                  <Select placeholder="请选择">{merchant.list.map(item => <Select.Option value={item.id} key={item.id}>{item.name}</Select.Option>)}</Select>
                )}
              </Form.Item>
              <Form.Item label="封面图片" {...formItemLayout}>
                <Button onClick={() => this.handleModalShow('cvImgBtn')} type="primary" icon="select" size="small">选择</Button>
                {!coverUrl ? null :
                  <Card
                    hoverable
                    bordered
                    cover={<img alt="" src={coverUrl.startsWith('http') ? coverUrl : `${ossHost}/${coverUrl}`} />}
                    style={{ width: 240, marginTop: 20 }}
                  >
                  </Card>}
              </Form.Item>
              <Form.Item label="背景图片" {...formItemLayout}>
                <Button onClick={() => this.handleModalShow('bgImgBtn')} type="primary" icon="select" size="small">选择</Button>
                {!bgUrl ? null :
                  <Card
                  hoverable
                  bordered
                  cover={<img alt="" src={bgUrl.startsWith('http') ? bgUrl : `${ossHost}/${bgUrl}`} />}
                  style={{ width: 240, marginTop: 20 }}
                >
                </Card>}
              </Form.Item>
              <Form.Item label="相关课" {...formItemLayout}>
                <Button onClick={() => this.handleModalShow('lessonBtn')} type="primary" size="small" icon="edit">编辑</Button>
              </Form.Item>
              <Form.Item wrapperCol={{ offset: 7, span: 12 }}>
                <Table
                  locale={{
                    emptyText: <span style={{ color: "#C6D0D6" }}>&nbsp;&nbsp;<Icon type="frown-o"/>
                      该课程下不包含任何课，请选择!</span>
                  }}
                  dataSource={subItemList}
                  columns={lessonTableColumns}
                  rowKey={record => record.id}
                  bordered
                  pagination={false}
                />
              </Form.Item>
              <Form.Item label="相关配套" {...formItemLayout}>
                <Button onClick={() => this.handleModalShow('supportBtn')} type="primary" size="small" icon="edit">编辑</Button>
              </Form.Item>
              <Form.Item wrapperCol={{ offset: 7, span: 12 }}>
                <Table
                  locale={{
                    emptyText: <span style={{ color: "#C6D0D6" }}>&nbsp;&nbsp;<Icon type="frown-o"/>
                      该课程暂无相关配套，请选择!</span>
                  }}
                  dataSource={supportList}
                  columns={supportTableColumns}
                  rowKey={record => record.id}
                  bordered
                  pagination={false}
                />
              </Form.Item>
              <Form.Item {...submitFormLayout} style={{ marginTop: 32 }}>
                <Button onClick={this.handlePageCancel}>取消</Button>
                <Button type="primary" style={{ marginLeft: 35 }} htmlType="submit">提交</Button>
              </Form.Item>
            </Form>
            {/*课的模态选择框*/}
            <LessonSelectSortModal
              rowKeyName="id"
              modalVisible={lessonModalVisible}
              width={600}
              onCancel={this.handleModalCancel}
              onOk={this.handleModalOk}
              onSearch={this.handleModalSearch}
              selTableData={subItemList || []}
              fsTableDataSource={lesson.list || []}
              fsTableLoading={lesson.listLoading}
              fsTablePagination={lesson.pagination}
              fsTableOnChange={this.handleModalTableChange}
            />
            {/*图片资源的模态选择框*/}
            <ResourceSelectModal
              rowKeyName="id"
              modalVisible={resourceModalVisible}
              width={600}
              onOk={this.handleModalOk}
              onCancel={this.handleModalCancel}
              onSearch={this.handleModalSearch}
              fsTableDataSource={resource.list || []}
              fsTableLoading={resource.listLoading}
              fsTablePagination={resource.pagination}
              fsTableOnChange={this.handleModalTableChange}
            />
            {/*周边的模态选择框*/}
            <SupportSelectSortModal
              rowKeyName="id"
              modalVisible={supportModalVisible}
              width={600}
              onCancel={this.handleModalCancel}
              onOk={this.handleModalOk}
              onSearch={this.handleModalSearch}
              selTableData={supportList || []}
              fsTableDataSource={support.list || []}
              fsTableLoading={support.listLoading}
              fsTablePagination={support.pagination}
              fsTableOnChange={this.handleModalTableChange}
            />
          </Card>
        </Spin>
      </PageHeaderLayout>
    );
  }
}
