import React, { PureComponent } from 'react';
import { Divider, Popover, Modal, Table, Form, Card, Button, Tag, Icon } from 'antd';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import queryString from 'query-string';
import PageHeaderLayout from '../../../layouts/PageHeaderLayout';
import FooterToolbar from '../../../components/FooterToolbar';
import DescriptionList from '../../../components/DescriptionList';
import NewPriceModal from './price';
import TagSelectModal from './tags';
import { pageSize, productType, Codes, ossHost } from '../../../utils/config';

const { Description } = DescriptionList;
const confirm = Modal.confirm;

@Form.create()
@connect(state => ({
  mproductDetail: state.mproductDetail,
  goodsModel: state.goodsModel,
  tagModel: state.tagModel,
}))
export default class MerchantProductEdit extends PureComponent {
  handleProductDetail = (data) => {
    const lessonTableColumns = [{
      title: '课编号',
      dataIndex: 'code',
      key: 'code',
      width: '40%',
    },{
      title: '课名称',
      dataIndex: 'title',
      key: 'title',
      width: '60%',
    }];
    const supportTableColumns = [{
      title: '配套编号',
      dataIndex: 'code',
      key: 'code',
      width: '40%',
    },{
      title: '配套名称',
      dataIndex: 'name',
      key: 'name',
      width: '60%',
    }];
    const trainingTableColumns = [{
      title: '位置',
      dataIndex: 'sort',
      key: 'sort',
      render: (text, record, index) => index + 1,
    },{
      title: '图片',
      dataIndex: 'path',
      key: 'path',
      render: (text, record) => (
        <Popover
            content={<img alt="" src={`${ossHost}/${text}`} width={350} />}
            title={record.name}
          >
            <img alt="" src={`${ossHost}/${text}`} width={70} />
          </Popover>
      ),
    }];
    const imgTableColumns = [{
      title: '位置',
      dataIndex: 'sort',
      key: 'sort',
      render: (text, record, index) => index + 1,
    },{
      title: '缩略图',
      dataIndex: 'path',
      key: 'path',
      render: (text, record) => (
          <Popover
            content={<img alt="" src={`${ossHost}/${text}`} width={350} />}
            title={record.name}
          >
            <img alt="" src={`${ossHost}/${text}`} width={70} />
          </Popover>
      ),
    }];
    const comboTableColumns = [{
      title: '编号',
      dataIndex: 'code',
      key: 'code',
      width: '30%',
    },{
      title: '名称',
      dataIndex: 'name',
      key: 'name',
      width: '40%',
    },{
      title: '类型',
      dataIndex: 'type',
      key: 'type',
      render: (text, record) => productType[text],
      width: '30%',
    }];
    if (!data) {
        return '暂无数据';
    } else if (data.type === Codes.CODE_COURSE) {
        return (
            <div>
                <DescriptionList size="large" col={2}>
                    <Description term="课程编号">{data.code}</Description>
                    <Description term="课程名称">{data.title}</Description>
                    <Description term="课程简介">{data.digest}</Description>
                    <Description term="课程详情">{data.detail}</Description>
                    <Description term="课程子标题">{data.subTitle}</Description>
                    <Description term="面包屑导航">{data.breadCrumb}</Description>
                    <Description term="课程封面图片">
                        {!data.coverUrl ? null :
                            <img
                                alt=""
                                src={data.coverUrl.startsWith('http') ? data.coverUrl : `${ossHost}/${data.coverUrl}`}
                                style={{ width: 200 }}
                            />
                        }
                    </Description>
                    <Description term="课程背景图片">
                        {!data.bgUrl ? null :
                            <img
                                alt=""
                                src={data.bgUrl.startsWith('http') ? data.bgUrl : `${ossHost}/${data.bgUrl}`}
                                style={{ width: 200 }}
                            />
                        }
                    </Description>
                    <Description term="课程内课列表">
                        <Table
                          locale={{
                            emptyText: <span style={{ color: "#C6D0D6" }}>&nbsp;&nbsp;<Icon type="frown-o"/>
                              该课程下不包含任何课!</span>
                          }}
                          dataSource={data.subItemList || []}
                          columns={lessonTableColumns}
                          rowKey={record => record.id}
                          bordered
                          pagination={false}
                        />
                    </Description>
                    <Description term="相关配套列表">
                        <Table
                          locale={{
                            emptyText: <span style={{ color: "#C6D0D6" }}>&nbsp;&nbsp;<Icon type="frown-o"/>
                              该课程无任何相关配套!</span>
                          }}
                          dataSource={data.supportList || []}
                          columns={supportTableColumns}
                          rowKey={record => record.id}
                          bordered
                          pagination={false}
                        />
                    </Description>
                </DescriptionList>
            </div>
        );
    } else if (data.type === Codes.CODE_SUPPORT) {
        return (
            <div>
                <DescriptionList size="large" col={2}>
                    <Description term="配套编号">{data.code}</Description>
                    <Description term="配套名称">{data.name}</Description>
                    <Description term="配套简介">{data.digest}</Description>
                    <Description term="配套详情">{data.detail}</Description>
                </DescriptionList>
                <DescriptionList size="large" col={1}>
                    <Description term="配套封面图片">
                        {!data.coverUrl ? null :
                            <img
                                alt=""
                                src={data.coverUrl.startsWith('http') ? data.coverUrl : `${ossHost}/${data.coverUrl}`}
                                style={{ width: 200 }}
                            />
                        }
                    </Description>
                </DescriptionList>
                <DescriptionList size="large" col={2} style={{ marginTop: 20 }}>
                    <Description term="配套图片列表">
                        <Table
                          locale={{
                            emptyText: <span style={{ color: "#C6D0D6" }}>&nbsp;&nbsp;<Icon type="frown-o"/>
                              该配套不包含任何图片!</span>
                          }}
                          dataSource={(data.imgList || []).map((item,index) => ({ id: index, path: item }))}
                          columns={imgTableColumns}
                          rowKey={record => record.id}
                          bordered
                          pagination={false}
                        />
                    </Description>
                    <Description term="相关配套列表">
                        <Table
                          locale={{
                            emptyText: <span style={{ color: "#C6D0D6" }}>&nbsp;&nbsp;<Icon type="frown-o"/>
                              该配套无任何相关配套!</span>
                          }}
                          dataSource={data.supportList || []}
                          columns={supportTableColumns}
                          rowKey={record => record.id}
                          bordered
                          pagination={false}
                        />
                    </Description>
                </DescriptionList>
            </div>
        );
    } else if (data.type === Codes.CODE_TRAINING) {
        return (
          <div>
            <DescriptionList size="large" col={1}>
              <Description term="师训编号">{data.code}</Description>
              <Description term="师训主题">{data.title}</Description>
              <Description term="活动时间">{data.dateDesc}</Description>
              <Description term="详情图片">
                <Table
                  simple
                  bordered
                  columns={trainingTableColumns}
                  dataSource={(data.imgList || []).map((item,index) => ({ id: index, path: item }))}
                  pagination={false}
                  rowKey={record => record.id}
                  locale={{
                    emptyText: <span style={{ color: "#C6D0D6" }}>&nbsp;&nbsp;<Icon type="frown-o"/>
                      该师训内暂无图片!</span>
                  }}
                />
              </Description>
            </DescriptionList>
          </div>
        );
    } else {
        return (
            <div>
                <DescriptionList size="large" col={1}>
                    <Description term="课程包编号">{data.code}</Description>
                    <Description term="课程包名称">{data.name}</Description>
                    <Description term="课程包内容">
                        <Table
                          simple
                          bordered
                          columns={comboTableColumns}
                          dataSource={data.products || []}
                          pagination={false}
                          rowKey={record => record.id}
                          locale={{
                            emptyText: <span style={{ color: "#C6D0D6" }}>&nbsp;&nbsp;<Icon type="frown-o"/>
                              该课程包内暂无相关内容!</span>
                          }}
                        />
                    </Description>
                </DescriptionList>
            </div>
        );
    }
  }

  handleAddPriceClick = () => {
    this.props.dispatch({
      type: 'goodsModel/showModal',
      payload: { operation: 'createItem' },
    });
  }

  handleEditPriceClick = (record) => {
    this.props.dispatch({
      type: 'goodsModel/showModal',
      payload: {
        operation: 'updateItem',
        goodsItem: record,
      },
    });
  }

  handleDelPriceClick = (record) => {
    const { dispatch, location } = this.props;
    const { search } = location;
    confirm({
      title: `您确定要删除此定价?`,
      onOk () {
        dispatch({
          type: 'goodsModel/removeItem',
          payload: { id: record.id },
          callback: () => {
            dispatch(routerRedux.push({
              pathname: '/goods/edit',
              search,
            }));
          }
        });
      },
    });
  }

  handleEditTagClick = () => {
    const { location } = this.props;
    const { search } = location;
    const { merchantId, pid } = queryString.parse(search);
    this.props.dispatch({
      type: 'goodsModel/showTagModal',
    });
    this.props.dispatch({
      type: 'tagModel/query',
      payload: {
        pageNo: 1,
        pageSize,
        merchantId,
      }
    });
  }

  handleTagModalCancel = () => {
    this.props.dispatch({
      type: 'goodsModel/hideTagModal',
    });
  }

  handleTagModalOk = (data) => {
    const { location } = this.props;
    const { search } = location;
    const { merchantId, pid } = queryString.parse(search);
    this.props.dispatch({
      type: 'goodsModel/bundleTags',
      payload: { pid, merchantId, tags: data.map(item => item.id) },
      callback: () => {
        this.props.dispatch(routerRedux.push({
          pathname: '/goods/edit',
          search,
        }));
      }
    });
  }

  handleTagModalSearch = (data) => {
    const { location } = this.props;
    const { search } = location;
    const { merchantId, pid } = queryString.parse(search);
    const newData = { ...data };
    if (newData.keyword) {
      newData[newData.field] = newData.keyword;
    }
    delete newData.field;
    delete newData.keyword;

    this.props.dispatch({
      type: 'tagModel/query',
      payload: {
        ...newData,
        pageNo: 1,
        pageSize,
        merchantId,
      }
    });
  }

  handlePageExit = () => {
    const { dispatch, mproductDetail } = this.props;
    const { filters } = mproductDetail;
    dispatch(routerRedux.push({
      pathname: '/goods',
      search: queryString.stringify(filters),
    }));
  }

  handleTagModalTableChange = (pagination, filterArgs, filters) => {
    const { location, dispatch } = this.props;
    const { search } = location;
    const { merchantId, pid } = queryString.parse(search);

    const newFilters = { ...filters };
    if (newFilters.keyword) {
      newFilters[newFilters.field] = newFilters.keyword;
    }
    delete newFilters.field;
    delete newFilters.keyword;

    const data = {
      ...newFilters,
      pageNo: pagination.current,
      pageSize: pagination.pageSize,
      merchantId,
    };

    Object.keys(data).map(key => data[key] ? null : delete data[key]);
    dispatch({ type: 'tagModel/query', payload: { ...data } });
  }

  render(){
    const { mproductDetail, goodsModel, tagModel, location } = this.props;
    const { currentItem, productItem } = mproductDetail;
    const { modalShow, tagModalShow, operation, goodsItem } = goodsModel;
    const { list, listLoading, pagination } = tagModel;
    const { goods, tags, type, code, name, merchantName } = currentItem;

    const { search } = location;
    const { merchantId, pid } = queryString.parse(search);

    const priceModalProps = {
      data: operation === 'createItem' ? {} : goodsItem,
      title: operation === 'createItem' ? '添加价格' : '编辑价格',
      visible: modalShow,
      maskClosable: false,
      onCancel: () => {
        this.props.dispatch({ type: 'goodsModel/hideModal' });
      },
      onSubmit: (data) => {
        const newData = { ...data, merchantId, pid, productType: type };
        this.props.dispatch({
          type: `goodsModel/${operation}`,
          payload: newData,
          callback: () => {
            this.props.dispatch(routerRedux.push({
              pathname: '/goods/edit',
              search,
            }));
          }
        });
      },
    };

    const listTableProps = {
      simple: true,
      bordered: true,
      pagination: false,
      rowKey: (record) => record.id,
      dataSource: goods,
      columns: [{
        title: '计价单位',
        dataIndex: 'chargeUnit',
        key: 'chargeUnit',
      },{
        title: '供应商价格(¥)',
        dataIndex: 'cpPrice',
        key: 'cpPrice',
      },{
        title: '渠道方价格(¥)',
        dataIndex: 'merchantPrice',
        key: 'merchantPrice',
      },{
        title: '终端价格(¥)',
        dataIndex: 'terminalPrice',
        key: 'terminalPrice',
      },{
        title: '操作',
        key: 'action',
        render: (text, record) => (
          <span>
            <a onClick={() => this.handleEditPriceClick(record)}>编辑</a>
            <Divider type="vertical" />
            <a onClick={() => this.handleDelPriceClick(record)}>删除</a>
          </span>
        ),
      }],
    };

    return (
      <PageHeaderLayout>
        <Card
          bordered={false}
          title="详情"
          style={{ marginBottom: 15 }}
        >{this.handleProductDetail(productItem)}
        </Card>
        <Card
          bordered={false}
          title={
            <div>定价
              <Divider type="vertical" />
              <Button onClick={this.handleAddPriceClick} size="small" type="primary">添加</Button>
            </div>
          }
          style={{ marginBottom: 15 }}
        >
          <Table { ...listTableProps }/>
          <NewPriceModal { ...priceModalProps } />
        </Card>
        <Card
          bordered={false}
          title={
            <div>标签
              <Divider type="vertical" />
              <Button onClick={this.handleEditTagClick} size="small" type="primary">选择</Button>
            </div>
          }
          style={{ marginBottom: 15 }}
        >
          {/*标签的模态选择框*/}
          <TagSelectModal
            rowKeyName="id"
            modalVisible={tagModalShow}
            selTableData={tags || []}
            fsTableDataSource={list}
            fsTableLoading={listLoading}
            fsTablePagination={pagination}
            onOk={this.handleTagModalOk}
            onCancel={this.handleTagModalCancel}
            onSearch={this.handleTagModalSearch}
            fsTableOnChange={this.handleTagModalTableChange}
          />
          {tags ? tags.map(item => <Tag key={item.id} color="#f50">{item.name}</Tag>) : null}
        </Card>
        <FooterToolbar>
          <Button onClick={this.handlePageExit} type="primary">
            返回产品列表
          </Button>
        </FooterToolbar>
      </PageHeaderLayout>
    );
  }
}
