import React, { PureComponent } from 'react';
import { routerRedux } from 'dva/router';
import queryString from 'query-string';
import { connect } from 'dva';
import PageHeaderLayout from '../../../layouts/PageHeaderLayout';
import BaseInfoCard from './baseInfo';
import RecommendList from './recommend';
import { Codes, pageSize } from '../../../utils/config';

@connect(state => ({
  mproduct: state.mproduct,
  merchantDetail: state.merchantDetail,
}))
export default class MerchantDetail extends PureComponent {
  state = { curTab: 'baseInfo' };

  handleTabChange = (key) => {
    this.setState({ curTab: key });
    if (key === 'recommend') {
      const { merchantDetail } = this.props;
      const { currentItem } = merchantDetail;
      const { id } = currentItem;
      this.props.dispatch({
        type: 'merchantDetail/queryMerchantRecommend',
        payload: { id },
      });
    }
  }

  handleRecommendAdjustClick = () => {
    const { merchantDetail } = this.props;
    const { currentItem } = merchantDetail;
    const { id } = currentItem;
    this.props.dispatch({ type: 'merchantDetail/showRecModal' });
    this.props.dispatch({
      type: 'mproduct/query',
      payload: {
        merchantId: id,
        pageNo: 1,
        pageSize,
      },
    });
  }

  handleRecommendModalCancel = () => {
    this.props.dispatch({ type: 'merchantDetail/hideRecModal' });
  }

  handleRecommendModalOk = (data) => {
    this.props.dispatch({
      type: 'merchantDetail/saveRecResult',
      payload: data,
    });
  }

  handleRecommendModalSearch = (data) => {
    const { merchantDetail } = this.props;
    const { currentItem } = merchantDetail;
    const { id } = currentItem;
    const newData = { ...data };
    if (newData.keyword) {
      newData[newData.field] = newData.keyword;
    }
    delete newData.field;
    delete newData.keyword;
    this.props.dispatch({
      type: 'mproduct/query',
      payload: {
        pageNo: 1,
        pageSize,
        merchantId: id,
        ...newData,
      },
    });
  }

  handleRecommendModalTableChange = (pagination, filterArgs, filters) => {
    const { dispatch, merchantDetail } = this.props;
    const { currentItem } = merchantDetail;
    const { id } = currentItem;
    const newFilters = { ...filters };
    if (newFilters.keyword) {
      newFilters[newFilters.field] = newFilters.keyword;
    }
    delete newFilters.field;
    delete newFilters.keyword;

    const data = {
      ...newFilters,
      pageNo: pagination.current,
      pageSize: pagination.pageSize,
      merchantId: id,
    };

    Object.keys(data).map(key => (data[key] ? null : delete data[key]));
    dispatch({ type: 'mproduct/query', payload: data });
  }

  handleRecommendPageCancel = () => {
    const { dispatch, merchantDetail } = this.props;
    const { filters } = merchantDetail;
    dispatch(
      routerRedux.push({
        pathname: '/merchant',
        search: queryString.stringify(filters),
      })
    );
    dispatch({
      type: 'merchantDetail/clearCache',
      payload: { recommend: [] },
    });
  }

  handleRecommendPageSubmit = () => {
    const { dispatch, merchantDetail } = this.props;
    const { filters, currentItem, recommend } = merchantDetail;
    const { id } = currentItem;
    const idList = recommend.map(item => item.pid);
    dispatch({
      type: 'merchantDetail/updateMerchantRecommend',
      payload: { merchantId: id, idList },
      callback: () => {
        dispatch(
          routerRedux.push({
            pathname: '/merchant',
            search: queryString.stringify(filters),
          })
        );
      },
    });
  }

  render() {
    const { dispatch, merchantDetail, mproduct } = this.props;
    const {
      itemLoading,
      currentItem,
      filters,
      operType,
      recommend,
      recModalShow,
      recLoading,
    } = merchantDetail;

    const { domain } = currentItem;

    let tabList;
    if (operType === 'update' && domain !== Codes.CODE_CP) {
      tabList = [{
        key: 'baseInfo',
        tab: '基础信息',
      }, {
        key: 'recommend',
        tab: '推荐位设置',
      }];
    }

    const baseInfoTabProps = {
      item: currentItem,
      loading: itemLoading,
      onCancel: () => {
        dispatch(
          routerRedux.push({
            pathname: '/merchant',
            search: queryString.stringify(filters),
          })
        );
        dispatch({
          type: 'merchantDetail/clearCache',
          payload: { currentItem: {} },
        });
      },
      onSubmit: (data) => {
        const { gmtCreated, gmtModified, ...rest } = currentItem;
        const newData = { ...rest, ...data };
        if (operType === 'create') {
          newData.status = Codes.CODE_NORMAL;
        }
        dispatch({
          type: `merchantDetail/${operType}`,
          payload: newData,
          callback: () => {
            dispatch(routerRedux.push({
              pathname: '/merchant',
              search: queryString.stringify(filters),
            }));
          },
        });
      },
    };
    const recommendTabProps = {
      rowKeyName: 'id',
      loading: recLoading,
      selTableData: recommend,
      modalVisible: recModalShow,
      fsTableDataSource: mproduct.list,
      fsTablePagination: mproduct.pagination,
      fsTableLoading: mproduct.listLoading,
      fsTableOnChange: this.handleRecommendModalTableChange,
      onOk: this.handleRecommendModalOk,
      onCancel: this.handleRecommendModalCancel,
      onSearch: this.handleRecommendModalSearch,
      modalShowController: this.handleRecommendAdjustClick,
      onPageCancel: this.handleRecommendPageCancel,
      onPageSubmit: this.handleRecommendPageSubmit,
    };

    const contentMap = {
      baseInfo: <BaseInfoCard {...baseInfoTabProps} />,
      recommend: <RecommendList {...recommendTabProps} />,
    };

    return (
      <PageHeaderLayout
        tabList={tabList}
        onTabChange={this.handleTabChange}
      >
        {contentMap[this.state.curTab]}
      </PageHeaderLayout>
    );
  }
}
