import React, { PureComponent } from 'react';
import { Card, Spin, Table, Button } from 'antd';
import SelectModal from '../../../components/SelectModal';
import styles from './recommend.less';
import { productType } from '../../../utils/config';

export default class RecommendList extends PureComponent {
  render() {
    const {
      loading,
      modalVisible,
      modalShowController,
      rowKeyName,
      selTableData,
      recModalShow,
      onCancel,
      onOk,
      onSearch,
      onPageCancel,
      onPageSubmit,
      ...fsTableOpts
    } = this.props;

    const modalProps = {
      title: '选择产品',
      maskClosable: false,
      visible: modalVisible,
      onCancel,
      onOk,
    };

    const searchProps = {
      searchField: 'name',
      searchKeyWord: '',
      searchSize: 'default',
      searchSelect: true,
      searchSelectOptions: [{
        value: 'name', name: '产品名称', mode: 'input',
      }, {
        value: 'code', name: '产品编号', mode: 'input',
      }],
      searchSelectProps: {
        defaultValue: 'name',
      },
      onSearch: (value) => {
        onSearch(value);
      },
    };

    const selTableProps = {
      operSort: true,
      operDel: true,
      tableClassName: styles.sTable,
      tablePagination: false,
      tableDataSource: selTableData,
      rowKeyName,
      tableColumns: [{
        title: '产品编号',
        dataIndex: 'code',
        key: 'code',
      }, {
        title: '产品名称',
        dataIndex: 'name',
        key: 'name',
      }, {
        title: '类型',
        dataIndex: 'type',
        key: 'type',
        render: text => productType[text],
      }],
    };

    const fsTableProps = {
      fsTableClassName: styles.fsTable,
      fsTableColumns: [{
        title: '产品编号',
        dataIndex: 'code',
        key: 'code',
      }, {
        title: '产品名称',
        dataIndex: 'name',
        key: 'name',
      }, {
        title: '类型',
        dataIndex: 'type',
        key: 'type',
        render: text => productType[text],
      }],
      ...fsTableOpts,
    };

    const columns = [{
      title: '产品编号',
      dataIndex: 'code',
      key: 'code',
    }, {
      title: '产品名称',
      dataIndex: 'name',
      key: 'name',
    }, {
      title: '类型',
      dataIndex: 'type',
      key: 'type',
      render: text => productType[text],
    }, {
      title: '位置',
      dataIndex: 'sort',
      key: 'sort',
      render: (text, record, index) => index + 1,
    }];

    return (
      <Spin spinning={loading}>
        <Card title={<Button onClick={modalShowController} type="primary" size="small" icon="edit">调整</Button>}>
          <Table
            bordered
            columns={columns}
            dataSource={selTableData}
            rowKey={record => record.id}
            pagination={false}
          />
          <Button onClick={onPageCancel} style={{ marginTop: 20 }}>退出</Button>
          <Button onClick={onPageSubmit} type="primary" style={{ marginLeft: 36, marginTop: 20 }}>保存</Button>
        </Card>
        {/* 渠道产品选择模态框 */}
        <SelectModal
          mode="multiple"
          {...modalProps}
          {...searchProps}
          {...selTableProps}
          {...fsTableProps}
        />
      </Spin>
    );
  }
}
