import React, { Component } from 'react';
import PropTypes from 'prop-types';
import queryString from 'query-string';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Card } from 'antd';
import TableList from './table';
import Search from './search';
import ModalForm from './modal';
import PageHeaderLayout from '../../../layouts/PageHeaderLayout';

@connect(state => ({resource: state.resource}))
export default class Gallery extends Component {

  render() {
    const { location, dispatch, resource } = this.props;
    location.query = queryString.parse(location.search);
    const { query, pathname } = location;
    const { field, keyword, ...filters } = query;
    const { list, listLoading, pagination, currentItem, itemLoading, modalVisible, modalType, signature } = resource;

    // 把携带的参数中空值项删除
    Object.keys(filters).map(key => { filters[key] ? null : delete filters[key] });
    // 如果搜索内容不为空则添加进filters中
    if (field && keyword) {
      filters.field = field;
      filters.keyword = keyword;
    }

    // 表头的搜索添加组件属性
    const searchProps = {
      field,
      keyword,
      onSearch: (payload) => {
        if (!payload.keyword.length) {
          delete payload.field;
          delete payload.keyword;
        }
        dispatch(routerRedux.push({
          pathname,
          search: queryString.stringify({
            ...payload
          })
        }));
      },
      onAdd: () => {
        dispatch({
          type: 'resource/showModal',
          payload: {
            modalType: 'create',
          },
        });
      }
    };

    // 列表组件的属性
    const listProps = {
      pagination,
      location,
      dataSource: list,
      loading: listLoading,
      onChange: (pagination, filterArgs) => {
        const getValue = obj => Object.keys(obj).map(key => obj[key]).join(',');
        const tableFilters = Object.keys(filterArgs).reduce((obj, key) => {
          const newObj = { ...obj };
          newObj[key] = getValue(filterArgs[key]);
          return newObj;
        }, {});
        const data = { ...filters, ...tableFilters };
        Object.keys(data).map(key => data[key] ? null : delete data[key]);

        dispatch(routerRedux.push({
          pathname,
          search: queryString.stringify({
            ...data,
            pageNo: pagination.current,
            pageSize: pagination.pageSize
          }),
        }));
      },
      onEditItem: (item) => {
        dispatch({
          type: 'resource/showModal',
          payload: {
            modalType: 'update',
            currentItem: item,
          },
          callback: () => {
            dispatch(
              routerRedux.push({
                pathname,
                search: queryString.stringify(filters),
              })
            );
          }
        })
      },
      // 删除一条数据后，刷新页面保持筛选数据不变
      onDeleteItem: (data) => {
        dispatch({
          type: 'resource/delete',
          payload: data,
          callback: () => {
            dispatch(
              routerRedux.push({
                pathname,
                search: queryString.stringify(filters),
              })
            );
          }
        });
      },
    };

    const modalProps = {
      item: currentItem,
      visible: modalVisible,
      maskClosable: false,
      signature,
      modalType,
      title: `${modalType == 'create' ? '添加图片' : '编辑图片'}`,
      wrapClassName: 'vertical-center-modal',
      onOk (data) {
        dispatch({
          type: `resource/${modalType}`,
          payload: data,
          callback: () => {
            dispatch(
              routerRedux.push({
                pathname,
                search: queryString.stringify(filters),
              })
            );
          },
        });
      },
      onCancel () {
        dispatch({
          type: 'resource/hideModal',
        });
      },
      onUpload (data) {
        dispatch({
          type: 'resource/updateItem',
          payload: data,
        });
      },
      onRemove (data) {
        dispatch({
          type: 'resource/updateItem',
          payload: data,
        });
      }
    };

    return (
      <PageHeaderLayout>
        <Card>
          <Search { ...searchProps } />
          <TableList { ...listProps } />
          <ModalForm { ...modalProps } />
        </Card>
      </PageHeaderLayout>
    );
  }
}
