import React, { PureComponent } from 'react';
import moment from 'moment';
import { Divider, Popover, Modal, Table, Menu, Icon, Badge } from 'antd';
import { statuses, Codes, ossHost } from '../../../utils/config'

export default class TableList extends PureComponent {

  handleDeleteItem = (record) => {
    const { onDeleteItem } = this.props;
    Modal.confirm({
      title: `您确定要删除该图片?`,
      okText: '确定',
      cancelText: '取消',
      onOk: () => onDeleteItem({id: record.id}),
    });
  }

  render() {
    const { onDeleteItem, onEditItem, pagination, ...tableProps } = this.props;

    const columns = [{
      title: '缩略图',
      dataIndex: 'url',
      key: 'url',
      render: (text, record) => (
        <Popover
          content={<img alt="" src={`${ossHost}/${record.path}`} width={250} />}
          title={record.name}
          placement="top"
        >
          <img alt="" src={`${ossHost}/${record.path}`} width={70} />
        </Popover>
      ),
      width: '14%',
    },{
      title: '图片编号',
      dataIndex: 'code',
      key: 'code',
      width: '22%',
    },{
      title: '图片名称',
      dataIndex: 'name',
      key: 'name',
      width: '22%',
    },{
      title: '图片大小(B)',
      dataIndex: 'size',
      key: 'size',
      width: '12%',
    },{
      title: '状态',
      dataIndex: 'status',
      key: 'status',
      render: (text, record) => {
        const statusMap = {[Codes.CODE_NORMAL]: 'success', [Codes.CODE_DELETE]: 'error'};
        return (<Badge status={statusMap[record.status]} text={statuses[record.status]} />);
      },
      width: '10%',
    },{
      title: '修改时间',
      dataIndex: 'gmtModified',
      key: 'gmtModified',
      render: (text, record) => (
        <div>{moment(text).format('YYYY-MM-DD')}</div>
      ),
      width: '12%',
    },{
      title: '操作',
      dataIndex: 'operation',
      key: 'operation',
      render: (text, record) => (
        <div>
          <a onClick={() => onEditItem(record)}>编辑</a>
          <Divider type="vertical" />
          <a onClick={() => this.handleDeleteItem(record)}>删除</a>
        </div>
      ),
      width: '8%',
    }];

    // 配置分页
    tableProps.pagination = !!pagination && { ...pagination, showSizeChanger: true, showQuickJumper: true, showTotal: total => `共 ${total} 条`};

    return (
      <Table
        simple
        bordered
        { ...tableProps }
        columns={columns}
        rowKey={record => record.id}
      />
    );
  }
}
