import React, { PureComponent } from 'react';
import moment from 'moment';
import { Table, Badge } from 'antd';
import { statuses, quality, Codes } from '../../../utils/config'

export default class TableList extends PureComponent {

  render() {
    const { onPlayVideo, pagination, ...tableProps } = this.props;

    const columns = [{
      title: '视频编号',
      dataIndex: 'code',
      key: 'code',
      width: '15%',
    },{
      title: '视频名称',
      dataIndex: 'name',
      key: 'name',
      width: '32%',
    },{
      title: '视频格式',
      dataIndex: 'format',
      key: 'format',
      width: '10%',
    },{
      title: '视频质量',
      dataIndex: 'quality',
      key: 'quality',
      render: (text, record) => quality[text],
      width: '10%',
    },{
      title: '状态',
      dataIndex: 'status',
      key: 'status',
      render: (text, record) => {
        const statusMap = {[Codes.CODE_NORMAL]: 'success', [Codes.CODE_DELETE]: 'error'};
        return (<Badge status={statusMap[record.status]} text={statuses[record.status]} />);
      },
      width: '10%',
    },{
      title: '添加时间',
      dataIndex: 'gmtCreated',
      key: 'gmtCreated',
      render: (text, record) => (
        <div>{moment(text).format('YYYY-MM-DD')}</div>
      ),
      width: '15%',
    },{
      title: '操作',
      dataIndex: 'operation',
      key: 'operation',
      render: (text, record) => (
        <div>
          <a onClick={() => onPlayVideo(record)}>播放</a>
        </div>
      ),
      width: '8%',
    }];

    // 配置分页
    tableProps.pagination = !!pagination && { ...pagination, showSizeChanger: true, showQuickJumper: true, showTotal: total => `共 ${total} 条`};
    // 视频返回的数据量每页不固定，会出现分页bug，这里截取与pageSize相等
    const pageSize = tableProps.pagination.pageSize;
    const newSource = tableProps.dataSource.slice(0, pageSize);

    return (
      <Table
        simple
        bordered
        { ...tableProps }
        columns={columns}
        dataSource={newSource}
        rowKey={record => record.id}
      />
    );
  }
}
