import React, { PureComponent } from 'react';
import { Badge } from 'antd';
import SelectModal from '../../../components/SelectModal';
import { Codes, productType, itemStatuses } from '../../../utils/config';

export default class ItemSortModal extends PureComponent {
  render() {
    const { itemList, modalVisible, rowKeyName, onCancel, onOk } = this.props;
    const modalProps = {
      title: '商品排序',
      maskClosable: false,
      visible: modalVisible,
      onCancel,
      onOk,
    };
    const selTableProps = {
      operSort: true,
      tablePagination: false,
      tableDataSource: itemList,
      rowKeyName: rowKeyName,
      tableColumns: [{
        title: '位置',
        dataIndex: 'sort',
        key: 'sort',
        render: (text, _, index) => index + 1,
      },{
        title: '产品名称',
        dataIndex: 'name',
        key: 'name',
      },{
        title: '产品状态',
        dataIndex: 'status',
        key: 'status',
        render: text => {
          const statusMap = {[Codes.CODE_NORMAL]: 'success', [Codes.CODE_DELETE]: 'error'};
          return (<Badge status={statusMap[text]} text={itemStatuses[text]} />);
        },
      },{
        title: '产品类型',
        dataIndex: 'type',
        key: 'type',
        render: text => (<span>{productType[text]}</span>),
      }],
    };

    return (
      <SelectModal
        mode="sort"
        { ...modalProps }
        { ...selTableProps }
      />
    );
  }
}
