import React, { Component } from 'react';
import moment from 'moment';
import queryString from 'query-string';
import { Divider, Table, Modal, Badge } from 'antd';
import { Codes, terminalStatuses } from '../../../utils/config';

export default class TerminalTableList extends Component {
  handleOperateItem = (record) => {
    const { onDeleteItem, onRecoverItem } = this.props;
    Modal.confirm({
      title: `您确定要${record.status === Codes.CODE_NORMAL ? '禁用' : '解禁'}该终端账号?`,
      onOk() {
        if (record.status === Codes.CODE_NORMAL) {
          onDeleteItem(record.id);
        } else {
          onRecoverItem({ id: record.id, status: Codes.CODE_NORMAL });
        }
      },
    });
  }
  handleUnBind = (record) => {
    const { onUnbindItem } = this.props;
    Modal.confirm({
      title: `您确定要解绑该终端账号?`,
      content: '解除绑定后登录将失效,请谨慎操作',
      okText: '确定',
      cancelText: '取消',
      onOk() {
        onUnbindItem({ id: record.id })
      },
    });
  }

  render() {
    const { onDeleteItem, onEditItem, curStatus, location, pagination, ...tableProps } = this.props;

    // 从url中提取查询参数
    location.query = queryString.parse(location.search);

    const columns = [{
      title: '终端编号',
      dataIndex: 'code',
      key: 'code',
      width: '14%',
    }, {
      title: '终端名称',
      dataIndex: 'name',
      key: 'name',
      width: '8%',
    }, {
      title: '校区',
      dataIndex: 'campusName',
      key: 'campusName',
      width: '30%',
    }, {
      title: '渠道名称',
      dataIndex: 'merchantId',
      key: 'merchantId',
      render: (text, record) => record.merchantName,
      width: '10%',
    }, {
      title: '状态',
      dataIndex: 'status',
      key: 'status',
      render: (text, record) => {
        const statusMap = { [Codes.CODE_NORMAL]: 'success', [Codes.CODE_DISABLE]: 'error' };
        return (<Badge status={statusMap[record.status]} text={terminalStatuses[record.status]} />);
      },
      filters: Object.keys(terminalStatuses).map(key =>
        ({ text: terminalStatuses[key], value: key })
      ),
      filterMultiple: false,
      filteredValue: [curStatus],
      width: '7%',
    }, {
      title: '修改时间',
      dataIndex: 'gmtModified',
      key: 'gmtModified',
      render: text => (
        <div>{moment(text).format('YYYY-MM-DD HH:mm:ss')}</div>
      ),
      width: '16%',
    }, {
      title: '操作',
      dataIndex: 'operation',
      key: 'operation',
      render: (text, record) => (
        <div>
          <a onClick={() => onEditItem(record)}>编辑</a>
          <Divider type="vertical" />
          <a onClick={() => this.handleOperateItem(record)}>{record.status === Codes.CODE_NORMAL ? '禁用' : '解禁'}</a>
          <Divider type="vertical" />
          <a onClick={() => this.handleUnBind(record)}>解绑</a>
        </div>
      ),
      width: '15%',
    }];

    columns.map((item) => {
      item.dataIndex === 'status' && !curStatus ? delete item.filteredValue : null;
    });

    // 配置分页
    tableProps.pagination = !!pagination && { ...pagination, showSizeChanger: true, showQuickJumper: true, showTotal: total => `共 ${total} 条` };

    return (
      <Table
        simple
        bordered
        {...tableProps}
        columns={columns}
        rowKey={record => record.id}
      />
    );
  }
}
