import React, { Component } from 'react';
import moment from 'moment';
import queryString from 'query-string';
import { Divider, Table, Modal, Badge } from 'antd';
import { Codes, statuses } from '../../../utils/config';

export default class WhitelistTable extends Component {
  handleOperateItem = (record) => {
    const { onDeleteItem } = this.props;
    Modal.confirm({
      title: `您确定要从白名单中移除该终端账号?`,
      cancelText: '取消',
      okText: '确定',
      onOk() {
        onDeleteItem(record.userId);
      }
    });
  }

  render() {
    const { onDeleteItem, onEditItem, curStatus, location, pagination, ...tableProps } = this.props;

    // 从url中提取查询参数
    location.query = queryString.parse(location.search);

    const columns = [{
      title: '终端编号',
      dataIndex: 'code',
      key: 'code',
      width: '14%',
    }, {
      title: '渠道名称',
      dataIndex: 'merchantId',
      key: 'merchantId',
      render: (text, record) => record.merchantName,
      width: '10%',
    }, {
      title: '起止日期',
      dataIndex: 'startTime',
      key: 'startTime',
      render: text => (
        <div>{moment(text).format('YYYY-MM-DD HH:mm:ss')}</div>
      ),
      width: '16%',
    }, {
      title: '截止日期',
      dataIndex: 'endTime',
      key: 'endTime',
      render: text => (
        <div>{text && moment(text).format('YYYY-MM-DD HH:mm:ss')}</div>
      ),
      width: '16%',
    }, {
      title: '状态',
      dataIndex: 'status',
      key: 'status',
      render: (text, record) => {
        const statusMap = { [Codes.CODE_NORMAL]: 'success', [Codes.CODE_DELETE]: 'error' };
        return (<Badge status={statusMap[record.status]} text={statuses[record.status]} />);
      },
      filters: Object.keys(statuses).map(key =>
        ({ text: statuses[key], value: key })
      ),
      filterMultiple: false,
      filteredValue: [curStatus],
      width: '7%',
    }, {
      title: '修改时间',
      dataIndex: 'gmtModified',
      key: 'gmtModified',
      render: text => (
        <div>{moment(text).format('YYYY-MM-DD HH:mm:ss')}</div>
      ),
      width: '16%',
    }, {
      title: '操作',
      dataIndex: 'operation',
      key: 'operation',
      render: (text, record) => (
        <div>
          <a onClick={() => onEditItem(record)}>编辑</a>
          <Divider type="vertical" />
          <a onClick={() => this.handleOperateItem(record)}>删除</a>
        </div>
      ),
      width: '10%',
    }];

    columns.map((item) => {
      item.dataIndex === 'status' && !curStatus ? delete item.filteredValue : null;
    });

    // 配置分页
    tableProps.pagination = !!pagination && { ...pagination, showSizeChanger: true, showQuickJumper: true, showTotal: total => `共 ${total} 条` };

    return (
      <Table
        simple
        bordered
        {...tableProps}
        columns={columns}
        rowKey={record => record.id}
      />
    );
  }
}
